codes = {'Afghanistan': 4,
         'Albania': 8,
         'Algeria': 12,
         'Angola': 24,
         'Antarctica': 10,
         'Argentina': 32,
         'Armenia': 51,
         'Australia': 36,
         'Austria': 40,
         'Azerbaijan': 31,
         'Bahamas': 44,
         'Bangladesh': 50,
         'Belarus': 112,
         'Belgium': 56,
         'Belize': 84,
         'Benin': 204,
         'Bhutan': 64,
         'Bolivia, Plurinational State of': 68,
         'Bosnia and Herzegovina': 70,
         'Botswana': 72,
         'Brazil': 76,
         'Brunei Darussalam': 96,
         'Bulgaria': 100,
         'Burkina Faso': 854,
         'Burundi': 108,
         'Cambodia': 116,
         'Cameroon': 120,
         'Canada': 124,
         'Central African Republic': 140,
         'Chad': 148,
         'Chile': 152,
         'China': 156,
         'Colombia': 170,
         'Congo': 178,
         'Congo, the Democratic Republic of the': 180,
         'Costa Rica': 188,
         "Cote d'Ivoire": 384,
         'Croatia': 191,
         'Cuba': 192,
         'Cyprus': 196,
         'Czech Republic': 203,
         'Denmark': 208,
         'Djibouti': 262,
         'Dominican Republic': 214,
         'Ecuador': 218,
         'Egypt': 818,
         'El Salvador': 222,
         'Equatorial Guinea': 226,
         'Eritrea': 232,
         'Estonia': 233,
         'Ethiopia': 231,
         'Falkland Islands (Malvinas)': 238,
         'Fiji': 242,
         'Finland': 246,
         'France': 250,
         'French Southern Territories': 260,
         'Gabon': 266,
         'Gambia': 270,
         'Georgia': 268,
         'Germany': 276,
         'Ghana': 288,
         'Greece': 300,
         'Greenland': 304,
         'Guatemala': 320,
         'Guinea': 324,
         'Guinea-Bissau': 624,
         'Guyana': 328,
         'Haiti': 332,
         'Honduras': 340,
         'Hungary': 348,
         'Iceland': 352,
         'India': 356,
         'Indonesia': 360,
         'Iran, Islamic Republic of': 364,
         'Iraq': 368,
         'Ireland': 372,
         'Israel': 376,
         'Italy': 380,
         'Jamaica': 388,
         'Japan': 392,
         'Jordan': 400,
         'Kazakhstan': 398,
         'Kenya': 404,
         "Korea, Democratic People's Republic of": 408,
         'Korea, Republic of': 410,
         'Kuwait': 414,
         'Kyrgyzstan': 417,
         "Lao People's Democratic Republic": 418,
         'Latvia': 428,
         'Lebanon': 422,
         'Lesotho': 426,
         'Liberia': 430,
         'Libya': 434,
         'Lithuania': 440,
         'Luxembourg': 442,
         'Macedonia, the former Yugoslav Republic of': 807,
         'Madagascar': 450,
         'Malawi': 454,
         'Malaysia': 458,
         'Mali': 466,
         'Mauritania': 478,
         'Mexico': 484,
         'Moldova, Republic of': 498,
         'Mongolia': 496,
         'Montenegro': 499,
         'Morocco': 504,
         'Mozambique': 508,
         'Myanmar': 104,
         'Namibia': 516,
         'Nepal': 524,
         'Netherlands': 528,
         'New Caledonia': 540,
         'New Zealand': 554,
         'Nicaragua': 558,
         'Niger': 562,
         'Nigeria': 566,
         'Norway': 578,
         'Oman': 512,
         'Pakistan': 586,
         'Panama': 591,
         'Papua New Guinea': 598,
         'Paraguay': 600,
         'Peru': 604,
         'Philippines': 608,
         'Poland': 616,
         'Portugal': 620,
         'Puerto Rico': 630,
         'Qatar': 634,
         'Romania': 642,
         'Russian Federation': 643,
         'Rwanda': 646,
         'Saudi Arabia': 682,
         'Senegal': 686,
         'Serbia': 688,
         'Sierra Leone': 694,
         'Slovakia': 703,
         'Slovenia': 705,
         'Solomon Islands': 90,
         'Somalia': 706,
         'South Africa': 710,
         'South Sudan': 728,
         'Spain': 724,
         'Sri Lanka': 144,
         'Sudan': 729,
         'Suriname': 740,
         'Swaziland': 748,
         'Sweden': 752,
         'Switzerland': 756,
         'Syrian Arab Republic': 760,
         'Taiwan, Province of China': 158,
         'Tajikistan': 762,
         'Tanzania, United Republic of': 834,
         'Thailand': 764,
         'Timor-Leste': 626,
         'Togo': 768,
         'Trinidad and Tobago': 780,
         'Tunisia': 788,
         'Turkey': 792,
         'Turkmenistan': 795,
         'Uganda': 800,
         'Ukraine': 804,
         'United Arab Emirates': 784,
         'United Kingdom': 826,
         'United States': 840,
         'Uruguay': 858,
         'Uzbekistan': 860,
         'Vanuatu': 548,
         'Venezuela, Bolivarian Republic of': 862,
         'Viet Nam': 704,
         'Western Sahara': 732,
         'Yemen': 887,
         'Zambia': 894,
         'Zimbabwe': 716}
